local T, VerifyStructure = ...
targetlist.API = {Version = 2}

-- tab_definition = {
-- 	title = "MyTab",
-- 	fgcolor = "R G B A" or number, -- (optional) foreground color of the tab; number requires .colorsource
-- 	bgcolor = "R G B A" or number, -- (optional) background color of the tab; number requires .colorsource
-- 	colorsource = table_reference, -- (optional) table reference for a list of color strings
-- 	columns = { -- Only 3 columns.  Width can be 0 and title can be empty.
-- 		{
-- 			title = "col1",
-- 			width = 0.2,
-- 		},
-- 		{
-- 			title = "col2",
-- 			width = 0.7,
-- 		},
-- 		{
-- 			title = "col3",
-- 			width = 0.1,
-- 		},
-- 	},
-- 	filters = { -- (optional)
-- 		link_filter_positions = true, -- (optional) if set filter and filter page positions will be shared between all filters on the tab; must define colors in the first filter page
-- 		{
-- 			title = "filter1",
-- 			fgcolor = "R G B A" or number, -- (optional) foreground color of the filter; will be used if no page color is set; number requires .colorsource
-- 			bgcolor = "R G B A" or number, -- (optional) background color of the filter; will be used if no page color is set; number requires .colorsource
-- 			pages = { -- (optional)
-- 				{
-- 					title = "page1",
-- 					fgcolor = "R G B A *" or number, -- (optional) foreground color of the filter page; number requires .colorsource
-- 					bgcolor = "R G B A *" or number, -- (optional) bakcground color of the filter page; number requires .colorsource
-- 				}
-- 			},
-- 		},
-- 	},
-- 	ChangeTarget = function(self, old_list_id, new_list_id) end,  -- (optional) called by MDI-TargetList when the target changes
-- 	ChangeFilter = function(self, filter_index, filter_page) end, -- (optional) called by MDI-TargetList when the filter changes
-- }

-- Event: TARGETLIST_TAB_CHANGED -- Triggered when the command /cycletab is executed
-- ChangeFilter() will be executed directly after TARGETLIST_TAB_CHANGED is triggered


function targetlist.API.IsActiveTab(identifier)
	return T.tab_identifiers[T.tab_index] == identifier
end

function targetlist.API.RegisterTab(tab_tbl)
	-- init_table: a table that should contain the title of the tab and the column definitions (title and width (%))
	--     This table will also be used as a unique identfier to correctly idenfity the source of data transmitted to the plugin in relation to the active tab.
	-- Returns false if too many tabs.  true otherwise
	VerifyStructure(tab_tbl)
	table.insert(T.tab_identifiers, tab_tbl)
	local filters = tab_tbl["filters"]
	if (filters) then
		local filter_positions = {index = 1}
		if (filters["link_filter_positions"]) then
			filter_positions["pages"] = 1
		else
			filter_positions["pages"] = {}
			for i in ipairs(filters) do
				filter_positions["pages"][i] = 1
			end
		end
		T.tab_filter_positions[tab_tbl] = filter_positions
	end

	--if (IsConnected()) then T:RebuildDisplay() end
	return true
end

function targetlist.API.UpdateDisplay(identifier, list)
	-- Only allow the plugin identifiying itself as the tab creator to update the tab display.
	-- identifiere:  the same tab_tbl used to originally register the tab.
	-- list:  sorted table to be displayed
	-- Return false if not a match to allow the caller to stop processing.
	if (T.tab_identifiers[T.tab_index] == identifier) then
		T.UI.display_container.UpdateDisplay(list, true)
		return true
	end
	return false
end

function targetlist.API.UpdateFilter(str)
	T.UI.filters[T.tab_filter_positions[T.tab_identifiers[T.tab_index]]["index"]]:SetTitle(str)
end

-- ----------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------

local function is_str_num(v)
	return type(v) == "string" or type(v) == "number"
end

VerifyStructure = function(tbl)
	assert(is_str_num(tbl.title),     "Tab Title is not a string or number.")

	-- Verity the columns are configured properly
	local cols = tbl.columns
	assert(cols and type(cols) == "table", "Column definition is missing or not a table.")
	assert(next(cols), "No columns were defined.")
	for ci, ct in ipairs(cols) do
		assert(is_str_num(ct.title), "Column:"..ci.." - Title is not a string or number.")
		assert(tonumber(ct.width),   "Column:"..ci.." - Width is not a number.")
	end

	-- Verify the filters are configured properly.  Filters are optional.
	local filts = tbl.filters
	if (filts) then
		assert(type(filts) == "table", "Filter definition is not a table.")
		assert(next(filts), "No filters were defined.")
		assert(#filts < 5, "Too many filters (max 4).")
		for fi, ft in ipairs(filts) do
			assert(is_str_num(ft.title), "Filter:"..fi.." - Title is not a string or number.")

			-- Verify the filter pages are configured properly.  Pages are optional.
			local pages = ft.pages
			if (pages) then
				assert(type(pages) == "table", "Filter page definition is not a table.")
				assert(next(pages), "No filter pages defined")
				for pi, pt in ipairs(pages) do
					assert(type(pt) == "table", "Filter:"..fi.." Page:"..pi.." - Missing definition table.")
					assert(is_str_num(pt.title), "Filter:"..fi.." Page:"..pi.." - Title is not a string or a number.")
				end
			end

		end
	end
	return true
end


--[=[
Version 2
 * Added .colorsource property and support for referencing colors by number



























]=]
