local config_name = "MDI TargetList"
local defaults = {
	fontscale             = {name = "Font Scale",            val = 1, Lval = 0, Uval = 2, dp = 2},
	target_color          = {name = "Highlight",             val = "65 255 65 255 *", Lval = 0, Uval = 255},
	list_count            = {name = "List Count",            val = 10, Lval = 1},
	name_width            = {name = "Name Width",            val = 0, Lval = 0},
	x                     = {name = "x",                     val = -1, Lval = 0, Uval = gkinterface.GetXResolution()},
	y                     = {name = "y",                     val = -1, Lval = 0, Uval = gkinterface.GetYResolution()},
	faction_filter        = {name = "Faction Filter",        val = 0},
	filter                = {name = "Filter",                val = 0},
	hide_turrets          = {name = "Hide Turrets",          val = 1},
	invert_faction_filter = {name = "Invert Faction Filter", val = 0},
	show_hive_only        = {name = "Show Hive Only",        val = 0},
	show_npc_ship_types   = {name = "Show NPC Ships",        val = 1},
	show_npc_faction      = {name = "Show NPC Faction",      val = 1},
	show_pc_ship_types    = {name = "Show PC Ships",         val = 0},
	show_all_players      = {name = "Show All Players",      val = 1},
	display_column_gap    = {name = "Column Gap",            val = 5},
	display_row_gap       = {name = "Row Gap",               val = 2},
	display_alpha         = {name = "Alpha",                 val = 255},
	display_drawcenter    = {name = "Draw Center",           val = 1},
	auto_pin_players      = {name = "Auto-Pin Players",      val = 0},
	pin_sort              = {name = "Pin Sort",              val = 1},
}

declare("targetlist", {
	version               = "4.1.1",
	config_name           = config_name,
--	ScanLock(true/false) -- use to set or test the scanlock flag to prevent TARGET_CHANGED from being processed.
})

T = {
	UI                    = {},
	Regions               = {},
	hide_turrets          = gkini.ReadInt   (config_name, defaults.hide_turrets.name,          defaults.hide_turrets.val) == 1,
	invert_faction_filter = gkini.ReadInt   (config_name, defaults.invert_faction_filter.name, defaults.invert_faction_filter.val) == 1,
	filter                = 1, -- legacy variable.  used to not rewrite core loops
	faction_filter        = 1, -- legacy variable.  used to not rewrite core loops
	fontscale             = gkini.ReadString(config_name, defaults.fontscale.name,             defaults.fontscale.val),
	current_target_color  = gkini.ReadString(config_name, defaults.target_color.name,          defaults.target_color.val),
	list_count            = gkini.ReadInt   (config_name, defaults.list_count.name,            defaults.list_count.val),
	name_width            = gkini.ReadInt   (config_name, defaults.name_width.name,            defaults.name_width.val),
	name_width_min        = 100,
	display_alpha         = gkini.ReadInt   (config_name, defaults.display_alpha.name,         defaults.display_alpha.val),
	display_column_gap    = gkini.ReadInt   (config_name, defaults.display_column_gap.name,    defaults.display_column_gap.val),
	display_row_gap       = gkini.ReadInt   (config_name, defaults.display_row_gap.name,       defaults.display_row_gap.val),
	display_drawcenter    = gkini.ReadInt   (config_name, defaults.display_drawcenter.name,         defaults.display_drawcenter.val) == 1,
	auto_pin_players      = gkini.ReadInt   (config_name, defaults.auto_pin_players.name,      defaults.auto_pin_players.val) == 1,
	pin_sort              = gkini.ReadInt   (config_name, defaults.pin_sort.name,              defaults.pin_sort.val),
	x                     = gkini.ReadInt   (config_name, defaults.x.name,                     defaults.x.val),
	y                     = gkini.ReadInt   (config_name, defaults.y.name,                     defaults.y.val),
	show_hive_only        = gkini.ReadInt   (config_name, defaults.show_hive_only.name,        defaults.show_hive_only.val) == 1,
	show_npc_ship_types   = gkini.ReadInt   (config_name, defaults.show_npc_ship_types.name,   defaults.show_npc_ship_types.val) == 1,
	show_npc_faction      = gkini.ReadInt   (config_name, defaults.show_npc_faction.name,      defaults.show_npc_faction.val) == 1,
	show_pc_ship_types    = gkini.ReadInt   (config_name, defaults.show_pc_ship_types.name,    defaults.show_pc_ship_types.val) == 1,
	show_all_players      = gkini.ReadInt   (config_name, defaults.show_all_players.name,      defaults.show_all_players.val) == 1,
	default_target_color  = defaults.target_color.val,
	list_offset           = 1,
	defense_timeout       = 15 * 60,
	aggressorlist         = {},
	defenselist           = {},
	pinlist               = {},
	playerlist            = {},
	shiplist              = {},
	id                    = "targetlist", -- used in MDI
	refresh_timer         = Timer(),
	refresh_delay         = 1000,
	current_target_listindex,
	current_target_nodeid,
	current_target_objectid,
	defaults = defaults,
	tab_identifiers = {}, -- holds init tables for tabs that are also used for identification when submitting a display update
	tab_index = 1, -- 0-based because of the use of modulo to handle tab cycling
	tab_window_size = 4,
	tab_window_offset = 0,
	tab_target_list = {}, -- holds the target list for the current tab
	tab_filter_positions_number_of = 0, -- changed based on the registered filters for the given tab
	tab_filter_positions = {},
}

--targetlist.T = T

if (T.x == -1) then T.x = nil end
if (T.y == -1) then T.y = nil end
if (T.name_width == 0) then T.name_width = nil end

console_print("MDI-TargetList loaded (v"..targetlist.version..").")

local function IsIUP(control) return control and pcall(iup.GetType, control) end

local function loadluafile(...)
	local f, err = loadfile((...))
	assert(f and type(f) == "function", err)
	return f(select(2, ...))
end

local function check_number(num, Lval, Uval, dp)
	-- verify a number is within the given range
	-- <num> the number to check
	-- <Lval>, <Uval>: Lower and Upper bounds of the number range; if missing, compares the number against itself
	-- [dp]:  how many decimal places to consider
	num = tonumber(num)
	Lval = tonumber(Lval) or num
	Uval = tonumber(Uval) or num
	if (num and Lval <= num and Uval >= num) then
		if (dp) then
			local decimals = tostring(num):match("%.(%d+)") or "0"
			if (decimals and decimals:len() <= dp) then
				return num -- entered number has no more decimal places than specified
			else
				return false -- too many decimal places
			end
		else
			return num -- non-decimal number is within range
		end
	end
	return false  -- you done fucked up
end

local Lock = false
function targetlist.ScanLock(state)
	if (state == nil) then return Lock end

	Lock = state
	if (isdeclared("targetless") and targetless) then
		targetless.api.radarlock = state
		targetless.var.lock = state
		targetless.var.scanlock = state
    end
    if (isdeclared("cargolist")) then
    end
    if (isdeclared("PR")) then
	end
end

function T:SaveSettings()
	gkini.WriteInt   (config_name, defaults.invert_faction_filter.name, self.invert_faction_filter and 1 or 0)
	--gkini.WriteInt   (config_name, defaults.filter.name,                self.filter)
	--gkini.WriteInt   (config_name, defaults.faction_filter.name,        self.faction_filter)
	gkini.WriteString(config_name, defaults.fontscale.name,             self.fontscale)
	gkini.WriteString(config_name, defaults.target_color.name,          self.current_target_color)
	gkini.WriteInt   (config_name, defaults.list_count.name,            self.list_count          or defaults.list_count.val)
	gkini.WriteInt   (config_name, defaults.name_width.name,            self.name_width          or defaults.name_width.val)
	gkini.WriteInt   (config_name, defaults.display_column_gap.name,    self.display_column_gap  or defaults.display_column_gap.val)
	gkini.WriteInt   (config_name, defaults.display_row_gap.name,       self.display_row_gap     or defaults.display_row_gap.val)
	gkini.WriteInt   (config_name, defaults.display_alpha.name,         self.display_alpha       or defaults.display_alpha.val)
	gkini.WriteInt   (config_name, defaults.display_drawcenter.name,    self.display_drawcenter  and 1 or 0)
	gkini.WriteInt   (config_name, defaults.x.name,                     self.x                   or defaults.x.val)
	gkini.WriteInt   (config_name, defaults.y.name,                     self.y                   or defaults.y.val)
	gkini.WriteInt   (config_name, defaults.auto_pin_players.name,      self.auto_pin_players    and 1 or 0)
	gkini.WriteInt   (config_name, defaults.pin_sort.name,              self.pin_sort            or defaults.pin_sort.val)
	gkini.WriteInt   (config_name, defaults.hide_turrets.name,          self.hide_turrets        and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_hive_only.name,        self.show_hive_only      and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_npc_ship_types.name,   self.show_npc_ship_types and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_npc_faction.name,      self.show_npc_faction    and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_pc_ship_types.name,    self.show_pc_ship_types  and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_all_players.name,      self.show_all_players    and 1 or 0)
end

local capship_type_names = {
	["Capella"] = true,
	["Goliath"] = true,
	["Heavy Assault Cruiser"] = true,
    ["Leviathan"] = true,
	["TPG Teradon Frigate"] = true,
    ["TPG Constellation Heavy Transport"] = true,
    ["Trident Light Frigate"] = true,
    ["Trident Type M"] = true,
}

local pin_sort_list = {
	"none",
	"name",
	"distance"
}

local function is_pinned(charID)
	return T.pinlist[charID] and 1 or 0
end

local function filter_players(factionid)
	if ((T.faction_filter) == 1) then
		return true
	elseif (not T.invert_faction_filter and factionid == (T.faction_filter - 1)) then
		return true
	elseif (T.invert_faction_filter and factionid ~= (T.faction_filter - 1)) then
		return true
	end
	return false
end

local function scan_player(charID)
	if (not charID) then return end
	local distance = GetRadarDistance(charID)
	local insert_player = false
	if (distance or T.show_all_players) then
		local name = GetPlayerName(charID)
		local ship_name = GetPrimaryShipNameOfPlayer(charID)
		local n, c = name:byte(1,2)
		local is_npc = n == 42 -- 42 = *; byte is faster than match
		local is_pc_cap = c == 32 -- 32 = space
		local factionID = GetPlayerFaction(charID)

		if     (T.hide_turrets and ship_name == "Turret") then
		    --do nothing

		elseif (T.filter == 1) then -- All
			insert_player = filter_players(factionID)

		elseif (T.filter == 2
			and (is_npc and (not T.show_hive_only or T.show_hive_only and ship_name == "Robot"))
			and	not is_pc_cap
		) then -- NPCs
			insert_player = filter_players(factionID)

		elseif (T.filter == 3 and (not is_npc or is_pc_cap))then -- PCs
			insert_player = filter_players(factionID)

		elseif (T.filter == 4) then -- Capships
			if (filter_players(factionID)) then
				if (capship_type_names[ship_name] or (ship_name == "Robot" and name:match("Queen"))) then
					insert_player = true
				end
			end
		end


		if (insert_player) then
			if (is_npc and not is_pc_cap) then
				if (T.show_npc_ship_types) then
					if (ship_name == "Turret") then
						name = "Turret "..name:match("Turret [%d-]*(%d)")
					elseif (ship_name ~= "Robot") then
						name = "*"..(ship_name or "Unknown")
					end
				end
				name = T.show_npc_faction and name.." ("..(FactionName[factionID] or "")..")" or name
			else
				if (T.show_pc_ship_types) then
					name = name.." - "..(ship_name or "Unknown")
				end
				if (T.auto_pin_players) then
					T.pinlist[charID] = true
				end
			end

			table.insert(T.shiplist, {
				charID = charID,
				factionID = factionID,
				name = name,
				distance = math.floor(distance or "9999"),
				capship = capship_type_names[ship_name] or ship_name == "Robot", -- Only queens have a ship named Robot and shields.  adding check to allow the shield display
			})
		end
	end

	return scan_player(table.remove(T.playerlist))
end

local function scan_sector()
	T.refresh_timer:Kill()

	T.playerlist = {}
	T.shiplist = {}

	ForEachPlayer(function(charID)
		if (charID ~= GetCharacterID()) then table.insert(T.playerlist, charID) end
	end)
	scan_player(table.remove(T.playerlist)) -- initiates a recursive tail call to iterate over the entire player list

	table.sort(T.shiplist, function(a, b)
		local A, B = is_pinned(a.charID) == 1, is_pinned(b.charID) == 1
		if (A and B) then
			if (T.pin_sort == 1) then
				return T.pinlist[a.charID] < T.pinlist[b.charID]
			else
				return a[pin_sort_list[T.pin_sort]] < b[pin_sort_list[T.pin_sort]]
			end
		elseif (A or B) then
			return A or not B
		end
		return a.distance < b.distance
	end)
	T.UI.display_container.UpdateDisplay(T.shiplist)
	T.refresh_timer:SetTimeout(T.refresh_delay, scan_sector)
end

function T.tab_filter_positions:GetPositions(tab_index)
	local cur_filter = self[T.tab_identifiers[tab_index]]
	if (not cur_filter) then return end

	local findex = cur_filter["index"]
	local fpage = cur_filter["pages"]
	fpage = type(fpage) == "table" and fpage[findex] or fpage
	return findex, fpage
end

function T.tab_filter_positions:SetPositions(tab_index, new_filter_index, new_page_index)
	local tab_tbl = T.tab_identifiers[tab_index]
	local cur_filter = self[tab_tbl]

	if (new_filter_index) then
		cur_filter["index"] = new_filter_index
		return new_filter_index

	elseif (new_page_index) then
		if (type(cur_filter["pages"]) == "table") then
			cur_filter["pages"][cur_filter["index"]] = new_page_index
		else
			cur_filter["pages"] = new_page_index
		end
		return new_page_index
	end
	error("Function called without arguments.")
end


loadluafile("api.lua", T) -- must be loaded first to allow the Targets tab to be registered so it can be created by other files
loadluafile("display.lua", T, scan_sector)
local change_target = loadluafile("commands.lua", T, scan_sector, check_number)
loadluafile("events.lua", T, IsIUP, scan_sector)
loadluafile("touch.lua", T, change_target)
loadluafile("help.lua", T)
loadluafile("options.lua", T, check_number)

-- filter pages below only serve to inform the plugin of how many filter pages there are per filter.  the titles aren't used.

T.Tab = {
	title = "Targets",
	--bgcolor = 3,
	colorsource = FactionColor_RGB,
	columns = {
		{title = "Health", width = 0},
		{title = "Name", width = 0},
		{title = "Distance", width = 0},
	},
	filters = {
		link_filter_positions = true,
		{
			title = "All",
			pages = {
				{
					title = "All",
					bgcolor = 0,
				},
				{
					title = "Itani",
					bgcolor = 1,
				},
				{
					title = "Serco",
					bgcolor = 2,
				},
				{
					title = "UIT",
					bgcolor = 3,
				},
			},
		},
		{
			title = "NPC",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
		{
			title = "PC",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
		{
			title = "Cap",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
	},
}
targetlist.API.RegisterTab(T.Tab)

-- T.Tab2 = {
-- 	title = "fab2",
-- 	fgcolor = "0 0 255",
-- 	bgcolor = "255 0 0",
-- 	columns = {{title = "tab2", width = 1},},
-- 	filters = {
-- 		{
-- 			title = "A little",
-- 			pages = {
-- 				{
-- 					title = "What",
-- 					bgcolor = "255 0 0",
-- 				},
-- 				{
-- 					title = "Why",
-- 					fgcolor = "255 0 0",
-- 				},
-- 			}
-- 		},
-- 		{
-- 			title = "Some",
-- 			fgcolor = "34 87 124",
-- 			bgcolor = "123 38 097",
-- 			pages = {
-- 				{title = "Something"},
-- 				{
-- 					title = "Not it",
-- 					fgcolor = "255 0 0",
--
-- 				},
-- 				{
-- 					title = "Yes it",
-- 					bgcolor = "0 255 0",
-- 				},
-- 			},
-- 		},
-- 		{
-- 			title = "None"
-- 		},
-- 	},
-- }
-- targetlist.API.RegisterTab(T.Tab2)
--
-- T.Tab3 = {
-- 	title = "fab3",
-- 	columns = {{title = "tab3", width = 1},},
-- }
-- targetlist.API.RegisterTab(T.Tab3)
--
-- T.Tab4 = {
-- 	title = "fab4",
-- 	columns = {{title = "tab4", width = 1},},
-- 	filters = {
-- 		link_filter_positions = true,
-- 		{
-- 			title = "All",
-- 			pages = {
-- 				{
-- 					title = "1",
-- 					bgcolor = "255 0 0",
-- 				},
-- 				{
-- 					title = "2",
-- 					bgcolor = "0 255 0",
-- 				},
-- 				{
-- 					title = "3",
-- 					bgcolor = "0 0 255",
-- 				},
-- 			},
-- 		},
-- 		{
-- 			title = "Some",
-- 			pages = {
-- 				{
-- 					title = "1",
-- 					bgcolor = "255 0 0",
-- 				},
-- 				{
-- 					title = "2",
-- 					bgcolor = "0 255 0",
-- 				},
-- 				{
-- 					title = "3",
-- 					bgcolor = "0 0 255",
-- 				},
-- 			},
-- 		},
-- 		{
-- 			title = "None",
-- 			pages = {
-- 				{
-- 					title = "1",
-- 					bgcolor = "255 0 0",
-- 				},
-- 				{
-- 					title = "2",
-- 					bgcolor = "0 255 0",
-- 				},
-- 				{
-- 					title = "3",
-- 					bgcolor = "0 0 255",
-- 				},
-- 			},
-- 		},
-- 	},
-- }
-- targetlist.API.RegisterTab(T.Tab4)
--
-- T.Tab5 = {
-- 	title = "fab5",
-- 	columns = {{title = "tab5", width = 1},},
-- 	filters = {
-- 		{
-- 			title = "All",
-- 		},
-- 	},
-- }
-- targetlist.API.RegisterTab(T.Tab5)
--
-- T.Tab6 = {
-- 	title = "fab6",
-- 	columns = {{title = "ftab6", width = 1},},
-- 	filters = {
-- 		{
-- 			title = "All",
-- 		},
-- 	},
-- }
-- targetlist.API.RegisterTab(T.Tab6)
